import { Context, Service, z } from 'cordis';
import { Awaitable, Dict } from 'cosmokit';
import { ReadableStream } from 'node:stream/web';
import type { Dispatcher, RequestInit, WebSocketInit } from 'undici';
declare module 'cordis' {
    interface Context {
        http: HTTP;
    }
    interface Intercept {
        http: HTTP.Intercept;
    }
    interface Events {
        'http/fetch'(this: HTTP, url: URL, init: RequestInit, config: HTTP.Config): Awaitable<globalThis.Response | undefined>;
        'http/config'(this: HTTP, config: HTTP.Config): void;
        'http/fetch-init'(this: HTTP, url: URL, init: RequestInit, config: HTTP.Config): void;
        'http/after-fetch'(this: HTTP, data: HTTP.AfterFetch): void;
        'http/websocket-init'(this: HTTP, url: URL, init: WebSocketInit, config: HTTP.Config): void;
    }
}
declare const kHTTPError: unique symbol;
declare class HTTPError extends Error {
    code?: HTTP.Error.Code | undefined;
    [kHTTPError]: boolean;
    response?: HTTP.Response;
    static is(error: any): error is HTTPError;
    constructor(message?: string, code?: HTTP.Error.Code | undefined);
}
export declare namespace HTTP {
    type Method = 'get' | 'GET' | 'delete' | 'DELETE' | 'head' | 'HEAD' | 'options' | 'OPTIONS' | 'post' | 'POST' | 'put' | 'PUT' | 'patch' | 'PATCH' | 'purge' | 'PURGE' | 'link' | 'LINK' | 'unlink' | 'UNLINK';
    interface ResponseTypes {
        json: any;
        text: string;
        stream: ReadableStream<Uint8Array>;
        blob: Blob;
        formdata: FormData;
        arraybuffer: ArrayBuffer;
    }
    interface Request1 {
        <K extends keyof ResponseTypes>(url: string, config: HTTP.RequestConfig & {
            responseType: K;
        }): Promise<ResponseTypes[K]>;
        <T = any>(url: string, config: HTTP.RequestConfig & {
            responseType: Decoder<T>;
        }): Promise<T>;
        <T = any>(url: string, config?: HTTP.RequestConfig): Promise<T>;
    }
    interface Request2 {
        <K extends keyof ResponseTypes>(url: string, data: any, config: HTTP.RequestConfig & {
            responseType: K;
        }): Promise<ResponseTypes[K]>;
        <T = any>(url: string, data: any, config: HTTP.RequestConfig & {
            responseType: Decoder<T>;
        }): Promise<T>;
        <T = any>(url: string, data?: any, config?: HTTP.RequestConfig): Promise<T>;
    }
    interface Intercept {
        baseURL?: string;
        headers?: Dict;
        timeout?: number;
        proxyAgent?: string;
    }
    interface Config extends Intercept {
    }
    interface RequestConfig extends Config {
        method?: Method;
        params?: Dict;
        data?: any;
        keepAlive?: boolean;
        redirect?: RequestRedirect;
        signal?: AbortSignal;
        responseType?: keyof ResponseTypes | Decoder;
        validateStatus?: (status: number) => boolean;
    }
    interface Response<T = any> {
        url: string;
        data: T;
        status: number;
        statusText: string;
        headers: Headers;
    }
    interface AfterFetch {
        url: URL;
        init: RequestInit;
        config: RequestConfig;
        response?: globalThis.Response;
        error?: any;
    }
    type Decoder<T = any> = (raw: globalThis.Response) => Awaitable<T>;
    type Error = HTTPError;
    namespace Error {
        type Code = 'ETIMEDOUT';
    }
}
export interface FileOptions {
    timeout?: number | string;
}
export interface HTTP {
    <K extends keyof HTTP.ResponseTypes>(url: string, config: HTTP.RequestConfig & {
        responseType: K;
    }): Promise<HTTP.Response<HTTP.ResponseTypes[K]>>;
    <T = any>(url: string | URL, config?: HTTP.RequestConfig): Promise<HTTP.Response<T>>;
    <T = any>(method: HTTP.Method, url: string | URL, config?: HTTP.RequestConfig): Promise<HTTP.Response<T>>;
    config: HTTP.Config;
    get: HTTP.Request1;
    delete: HTTP.Request1;
    patch: HTTP.Request2;
    post: HTTP.Request2;
    put: HTTP.Request2;
}
export declare class HTTP extends Service {
    config: HTTP.Config;
    static Error: typeof HTTPError;
    static undici: typeof import('undici');
    static Config: z<HTTP.Config>;
    static Intercept: z<HTTP.Config>;
    isError: typeof HTTPError.is;
    private _decoders;
    private _proxies;
    constructor(ctx: Context, config?: HTTP.Config);
    get undici(): typeof import("undici");
    static mergeConfig: (target: HTTP.Config, source?: HTTP.Config) => {
        headers: {
            [x: string]: any;
        };
        baseURL?: string;
        timeout?: number;
        proxyAgent?: string;
    };
    decoder<K extends keyof HTTP.ResponseTypes>(type: K, decoder: HTTP.Decoder<HTTP.ResponseTypes[K]>): () => Promise<void>;
    proxy(name: string[], factory: (url: URL) => Dispatcher): () => Promise<void>;
    extend(config?: HTTP.Config): any;
    resolveConfig(init?: HTTP.RequestConfig): HTTP.RequestConfig;
    resolveURL(url: string | URL, config: HTTP.RequestConfig, isWebSocket?: boolean): URL;
    defaultDecoder(response: Response): Promise<any>;
    [Service.invoke](...args: any[]): Promise<HTTP.Response<any>>;
    head(url: string, config?: HTTP.Config): Promise<Headers>;
    /** @deprecated use `ctx.http()` instead */
    axios<T = any>(config: {
        url: string;
    } & HTTP.RequestConfig): Promise<HTTP.Response<T>>;
    axios<T = any>(url: string, config?: HTTP.RequestConfig): Promise<HTTP.Response<T>>;
    ws(url: string | URL, _config?: HTTP.Config): import("undici").WebSocket;
    isLocal(url: string): Promise<boolean>;
}
export default HTTP;
