var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });

// src/index.ts
import { Context, Service, z } from "cordis";
import { Binary, defineProperty, isNullable } from "cosmokit";
import { fetchFile, lookup } from "@cordisjs/plugin-http/adapter";
import { createRequire } from "node:module";

// src/utils.ts
var bogonV4 = [
  "0.0.0.0/8",
  // RFC 1122 'this' network
  "10.0.0.0/8",
  // RFC 1918 private space
  "100.64.0.0/10",
  // RFC 6598 Carrier grade nat space
  "127.0.0.0/8",
  // RFC 1122 localhost
  "169.254.0.0/16",
  // RFC 3927 link local
  "172.16.0.0/12",
  // RFC 1918 private space
  "192.0.2.0/24",
  // RFC 5737 TEST-NET-1
  "192.88.99.0/24",
  // RFC 7526 6to4 anycast relay
  "192.168.0.0/16",
  // RFC 1918 private space
  "198.18.0.0/15",
  // RFC 2544 benchmarking
  "198.51.100.0/24",
  // RFC 5737 TEST-NET-2
  "203.0.113.0/24",
  // RFC 5737 TEST-NET-3
  "224.0.0.0/4",
  // multicast
  "240.0.0.0/4"
  // reserved
];
var bogonV6 = [
  "::/8",
  // RFC 4291 IPv4-compatible, loopback, et al
  "0100::/64",
  // RFC 6666 Discard-Only
  "2001:2::/48",
  // RFC 5180 BMWG
  "2001:10::/28",
  // RFC 4843 ORCHID
  "2001:db8::/32",
  // RFC 3849 documentation
  "2002::/16",
  // RFC 7526 6to4 anycast relay
  "3ffe::/16",
  // RFC 3701 old 6bone
  "fc00::/7",
  // RFC 4193 unique local unicast
  "fe80::/10",
  // RFC 4291 link local unicast
  "fec0::/10",
  // RFC 3879 old site local unicast
  "ff00::/8"
  // RFC 4291 multicast
];
function parseIPv4(ip) {
  return ip.split(".").reduce((a, b) => (a << 8n) + BigInt(b), 0n);
}
__name(parseIPv4, "parseIPv4");
function parseIPv6(ip) {
  const exp = ip.indexOf("::");
  let num = 0n;
  if (exp !== -1 && exp !== 0) {
    ip.slice(0, exp).split(":").forEach((piece, i) => {
      num |= BigInt(`0x${piece}`) << BigInt((7 - i) * 16);
    });
  }
  if (exp === ip.length - 2) {
    return num;
  }
  const rest = exp === -1 ? ip : ip.slice(exp + 2);
  const v4 = rest.includes(".");
  const pieces = rest.split(":");
  let start = 0;
  if (v4) {
    start += 2;
    const [addr] = pieces.splice(-1, 1);
    num |= parseIPv4(addr);
  }
  pieces.reverse().forEach((piece, i) => {
    num |= BigInt(`0x${piece}`) << BigInt((start + i) * 8);
  });
  return num;
}
__name(parseIPv6, "parseIPv6");
function isLocalAddress({ address, family }) {
  if (family !== 4 && family !== 6) return false;
  const { bogons, length, parse } = family === 4 ? { bogons: bogonV4, length: 32, parse: parseIPv4 } : { bogons: bogonV6, length: 128, parse: parseIPv6 };
  const num = parse(address);
  for (const bogon of bogons) {
    const [prefix, cidr] = bogon.split("/");
    const mask = (1n << BigInt(cidr)) - 1n << BigInt(length - +cidr);
    if ((num & mask) === parse(prefix)) return true;
  }
  return false;
}
__name(isLocalAddress, "isLocalAddress");

// src/index.ts
var kHTTPError = Symbol.for("cordis.http.error");
var HTTPError = class extends Error {
  constructor(message, code) {
    super(message);
    this.code = code;
  }
  static {
    __name(this, "HTTPError");
  }
  [kHTTPError] = true;
  response;
  static is(error) {
    return !!error?.[kHTTPError];
  }
};
function encodeRequest(data) {
  if (data instanceof URLSearchParams) return [null, data];
  if (data instanceof ArrayBuffer) return [null, data];
  if (ArrayBuffer.isView(data)) return [null, data];
  if (data instanceof Blob) return [null, data];
  if (data instanceof FormData) return [null, data];
  return ["application/json", JSON.stringify(data)];
}
__name(encodeRequest, "encodeRequest");
var HTTP = class _HTTP extends Service {
  constructor(ctx, config = {}) {
    super(ctx, "http");
    this.config = config;
    this.decoder("json", (raw) => raw.json());
    this.decoder("text", (raw) => raw.text());
    this.decoder("blob", (raw) => raw.blob());
    this.decoder("arraybuffer", (raw) => raw.arrayBuffer());
    this.decoder("formdata", (raw) => raw.formData());
    this.decoder("stream", (raw) => raw.body);
    this.proxy(["http", "https"], (url) => {
      return new this.undici.ProxyAgent(url.href);
    });
    this.ctx.on("http/fetch", (url, init, config2) => {
      return this.undici.fetch(url, init);
    });
    this.ctx.on("http/fetch", (url, init, config2) => {
      if (url.protocol !== "file:") return;
      if (init.method !== "GET") {
        return new Response(null, { status: 405, statusText: "Method Not Allowed" });
      }
      return fetchFile(url, init);
    }, { prepend: true });
    this.ctx.on("http/fetch", (url, init, config2) => {
      const capture = /^data:([\w/.+-]+);base64,(.*)$/.exec(url.href);
      if (!capture) return;
      if (init.method !== "GET") {
        return new Response(null, { status: 405, statusText: "Method Not Allowed" });
      }
      const [, type, base64] = capture;
      return new Response(Binary.fromBase64(base64), {
        status: 200,
        statusText: "OK",
        headers: { "Content-Type": type }
      });
    }, { prepend: true });
  }
  static {
    __name(this, "HTTP");
  }
  static Error = HTTPError;
  static undici;
  static {
    const require2 = createRequire(import.meta.url);
    try {
      if (process.execArgv.includes("--expose-internals")) {
        this.undici = require2("internal/deps/undici/undici");
      } else {
        this.undici = require2("undici");
      }
    } catch {
    }
    for (const method of ["get", "delete"]) {
      defineProperty(_HTTP.prototype, method, async function(url, config) {
        const response = await this(url, { method, ...config });
        return response.data;
      });
    }
    for (const method of ["patch", "post", "put"]) {
      defineProperty(_HTTP.prototype, method, async function(url, data, config) {
        const response = await this(url, { method, data, ...config });
        return response.data;
      });
    }
  }
  static Config = z.object({
    timeout: z.natural().role("ms").description("等待请求的最长时间。"),
    keepAlive: z.boolean().description("是否保持连接。"),
    proxyAgent: z.string().description("代理服务器地址。")
  });
  static Intercept = z.object({
    baseURL: z.string().description("基础 URL。"),
    timeout: z.natural().role("ms").description("等待请求的最长时间。"),
    keepAlive: z.boolean().description("是否保持连接。"),
    proxyAgent: z.string().description("代理服务器地址。")
  });
  isError = HTTPError.is;
  _decoders = /* @__PURE__ */ Object.create(null);
  _proxies = /* @__PURE__ */ Object.create(null);
  get undici() {
    if (_HTTP.undici) return _HTTP.undici;
    throw new Error("please install `undici`");
  }
  static mergeConfig = /* @__PURE__ */ __name((target, source) => ({
    ...target,
    ...source,
    headers: {
      ...target?.headers,
      ...source?.headers
    }
  }), "mergeConfig");
  decoder(type, decoder) {
    return this.ctx.effect(() => {
      this._decoders[type] = decoder;
      return () => delete this._decoders[type];
    });
  }
  proxy(name, factory) {
    return this.ctx.effect(() => {
      for (const key of name) {
        this._proxies[key] = factory;
      }
      return () => {
        for (const key of name) {
          delete this._proxies[key];
        }
      };
    });
  }
  extend(config = {}) {
    return this[Service.extend]({
      config: _HTTP.mergeConfig(this.config, config)
    });
  }
  resolveConfig(init) {
    let result = { headers: {}, ...this.config };
    this.ctx.emit(this, "http/config", result);
    let intercept = this.ctx[Context.intercept];
    while (intercept) {
      result = _HTTP.mergeConfig(result, intercept.http);
      intercept = Object.getPrototypeOf(intercept);
    }
    result = _HTTP.mergeConfig(result, init);
    return result;
  }
  resolveURL(url, config, isWebSocket = false) {
    try {
      url = new URL(url, config.baseURL);
    } catch (error) {
      throw new TypeError(`Invalid URL: ${url}`);
    }
    if (isWebSocket) url.protocol = url.protocol.replace(/^http/, "ws");
    for (const [key, value] of Object.entries(config.params ?? {})) {
      if (isNullable(value)) continue;
      url.searchParams.append(key, value);
    }
    return url;
  }
  defaultDecoder(response) {
    const type = response.headers.get("Content-Type");
    if (type?.startsWith("application/json")) {
      return response.json();
    } else if (type?.startsWith("text/")) {
      return response.text();
    } else {
      return response.arrayBuffer();
    }
  }
  async [Service.invoke](...args) {
    let method;
    if (typeof args[1] === "string" || args[1] instanceof URL) {
      method = args.shift();
    }
    const config = this.resolveConfig(args[1]);
    const url = this.resolveURL(args[0], config);
    method ??= config.method ?? "GET";
    const controller = new AbortController();
    if (config.signal) {
      if (config.signal.aborted) {
        throw config.signal.reason;
      }
      config.signal.addEventListener("abort", () => {
        controller.abort(config.signal.reason);
      });
    }
    const dispose = this.ctx.effect(() => {
      const timer = config.timeout && setTimeout(() => {
        controller.abort(new HTTPError("request timeout", "ETIMEDOUT"));
      }, config.timeout);
      return () => {
        clearTimeout(timer);
      };
    });
    controller.signal.addEventListener("abort", () => dispose());
    try {
      const headers = new Headers(config.headers);
      const init = {
        method,
        headers,
        body: config.data,
        keepalive: config.keepAlive,
        redirect: config.redirect,
        signal: controller.signal
      };
      if (config.data && typeof config.data === "object") {
        const [type, body] = encodeRequest(config.data);
        init.body = body;
        if (type && !headers.has("Content-Type")) {
          headers.append("Content-Type", type);
        }
      }
      if (config.proxyAgent) {
        const proxyURL = new URL(config.proxyAgent);
        const factory = this._proxies[proxyURL.protocol.slice(0, -1)];
        if (!factory) throw new Error(`Cannot resolve proxy agent ${proxyURL}`);
        init.dispatcher = factory(proxyURL);
      }
      this.ctx.emit(this, "http/fetch-init", url, init, config);
      const raw = await this.ctx.serial(this, "http/fetch", url, init, config).catch((cause) => {
        this.ctx.emit(this, "http/after-fetch", { url, init, config, error: cause });
        if (_HTTP.Error.is(cause)) throw cause;
        const error = new _HTTP.Error(`fetch ${url} failed`);
        error.cause = cause;
        throw error;
      });
      this.ctx.emit(this, "http/after-fetch", { url, init, config, response: raw });
      const response = {
        data: null,
        url: raw.url,
        status: raw.status,
        statusText: raw.statusText,
        headers: raw.headers
      };
      const validateStatus = config.validateStatus ?? ((status) => status < 400);
      if (!validateStatus(raw.status)) {
        const error = new _HTTP.Error(raw.statusText);
        error.response = response;
        try {
          response.data = await this.defaultDecoder(raw);
        } catch {
        }
        throw error;
      }
      if (config.responseType) {
        let decoder;
        if (typeof config.responseType === "function") {
          decoder = config.responseType;
        } else {
          decoder = this._decoders[config.responseType];
          if (!decoder) {
            throw new TypeError(`Unknown responseType: ${config.responseType}`);
          }
        }
        response.data = await decoder(raw);
      } else {
        response.data = await this.defaultDecoder(raw);
      }
      return response;
    } finally {
      dispose();
    }
  }
  async head(url, config) {
    const response = await this(url, { method: "HEAD", ...config });
    return response.headers;
  }
  axios(...args) {
    this.ctx.emit(this.ctx, "internal/warning", "ctx.http.axios() is deprecated, use ctx.http() instead");
    if (typeof args[0] === "string") {
      return this(args[0], args[1]);
    } else {
      return this(args[0].url, args[0]);
    }
  }
  ws(url, _config) {
    const config = this.resolveConfig(_config);
    url = this.resolveURL(url, config, true);
    const headers = new Headers(config.headers);
    const init = {
      headers
    };
    if (config.proxyAgent) {
      const proxyURL = new URL(config.proxyAgent);
      const factory = this._proxies[proxyURL.protocol.slice(0, -1)];
      if (!factory) throw new Error(`Cannot resolve proxy agent ${proxyURL}`);
      init.dispatcher = factory(proxyURL);
    }
    this.ctx.emit(this, "http/websocket-init", url, init, config);
    const socket = new this.undici.WebSocket(url, init);
    const dispose = this.ctx.on("dispose", () => {
      socket.close(1e3, "context disposed");
    });
    socket.addEventListener("close", () => {
      dispose();
    });
    return socket;
  }
  async isLocal(url) {
    let { hostname, protocol } = new URL(url);
    if (protocol !== "http:" && protocol !== "https:") return true;
    if (/^\[.+\]$/.test(hostname)) {
      hostname = hostname.slice(1, -1);
    }
    try {
      const address = await lookup(hostname);
      return isLocalAddress(address);
    } catch {
      return false;
    }
  }
};
var index_default = HTTP;
export {
  HTTP,
  index_default as default
};
